Traits
- GetOr
Insert - A trait for enum types that support
.insert(x)method which sets the enum variant to something unique forxand returns a mutable reference to the value.
.insert(x) method which sets the enum variant to
something unique for x and returns a mutable reference to the value.